<?php

namespace Drupal\untranslated_404\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\language\Entity\ContentLanguageSettings;

/**
 * Deny access if an en entity doesn't have a translation.
 */
class HasTranslation implements AccessInterface {

  /**
   * The access result reason.
   */
  public const ACCESS_RESULT_REASON = 'Entity has no translation';

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(LanguageManagerInterface $language_manager) {
    $this->languageManager = $language_manager;
  }

  /**
   * Deny access if the route match refers to an untranslated entity.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(RouteMatchInterface $route_match): AccessResultInterface {
    // Allow access if no translatable entity has been specified.
    $parameters = $route_match->getParameters()->all();
    $entity = reset($parameters);

    if (!$entity instanceof TranslatableInterface) {
      return AccessResult::allowed();
    }

    // Allow access if the entity is in an untranslatable language.
    $untranslatable_languages = [
      LanguageInterface::LANGCODE_NOT_SPECIFIED,
      LanguageInterface::LANGCODE_NOT_APPLICABLE,
    ];

    if (in_array($entity->language()->getId(), $untranslatable_languages, TRUE)) {
      return AccessResult::allowed();
    }

    // Allow access if content translation isn't enabled.
    $config = ContentLanguageSettings::loadByEntityTypeBundle($entity->getEntityTypeId(), $entity->bundle());
    $content_translation = $config->getThirdPartySettings('content_translation');

    if (empty($content_translation['enabled'])) {
      return AccessResult::allowed()->addCacheableDependency($config);
    }

    // Allow access if a translation exists.
    $langcode = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();

    if ($entity->hasTranslation($langcode)) {
      $result = AccessResult::allowed();
    }
    else {
      $result = AccessResult::forbidden(self::ACCESS_RESULT_REASON);
    }

    return $result
      ->addCacheContexts(['languages:language_content'])
      ->addCacheableDependency($config);
  }

}
