<?php

namespace Drupal\untranslated_404\EventSubscriber;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\EventSubscriber\HttpExceptionSubscriberBase;
use Drupal\Core\Routing\RouteBuildEvent;
use Drupal\Core\Routing\RoutingEvents;
use Drupal\untranslated_404\Access\HasTranslation;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Event subscriber to return a 404 if an entity isn't translated.
 */
class NotFoundIfNoTranslationSubscriber extends HttpExceptionSubscriberBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Class constructor
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = parent::getSubscribedEvents();
    $events[RoutingEvents::ALTER][] = 'onRouteAlter';

    return $events;
  }

  /**
   * Act on the routes being build.
   *
   * @param \Drupal\Core\Routing\RouteBuildEvent $event
   *   The route build event.
   */
  public function onRouteAlter(RouteBuildEvent $event): void {
    foreach ($this->entityTypeManager->getDefinitions() as $definition) {
      if (!is_subclass_of($definition->getClass(), ContentEntityInterface::class)) {
        continue;
      }

      $event->getRouteCollection()
        ->get("entity.{$definition->id()}.canonical")
        ?->setRequirement('_untranslated_404_has_translation', 'TRUE');
    }
  }

  /**
   * Change a 403 to a 404 if the entity isn't translated.
   *
   * @param \Symfony\Component\HttpKernel\Event\ExceptionEvent $event
   *   The exception event.
   */
  public function on403(ExceptionEvent $event): void {
    $exception = $event->getThrowable();

    if ($exception->getMessage() === HasTranslation::ACCESS_RESULT_REASON) {
      $event->setThrowable(new NotFoundHttpException($exception->getMessage(), $exception));
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getHandledFormats(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  protected static function getPriority(): int {
    return 100;
  }

}
